/***************************************************************************
 *
 * Copyright (C) 2001 International Business Machines
 * All rights reserved.
 *
 * This file is part of the GPFS mmfslinux kernel module.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer. 
 *  2. Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution. 
 *  3. The name of the author may not be used to endorse or promote products 
 *     derived from this software without specific prior written
 *     permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *************************************************************************** */
/* @(#)00       1.10  src/avs/fs/mmfs/ts/kernext/gpl-linux/acl.c, mmfs, avs_rcs2b, rcs2bs001a 11/10/03 08:36:18 */

#ifdef NFS4_CONFIG_FS_EXT_ATTR
#include <Shark-gpl.h>

#include <linux/string.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/smp_lock.h>

#ifdef NFS4_CONFIG_FS_POSIX_ACL
#include <linux_posix_acl.h>
#endif

#include <verdep.h>
#include <cxiTSFattr.h>
#include <cxiAclUser.h>

#define PERM_MASK(perm) perm&(LINUX_ACL_READ|LINUX_ACL_WRITE|LINUX_ACL_EXECUTE)

#ifdef NFS4_CONFIG_FS_POSIX_ACL
/* move data from ACL to posix_acl_t */
void
ACL_2_posix_acl_t(ACL *tmpaclP, posix_acl_t *aclP)
{
  int i,count;
  acl_entry_t *aceP;

  /* Get the address of the first extended acl entry */
/* ??? must fix struct ACL before it can be used in C code
  aceP = ACL_ENTRY_START(tmpaclP,tmpaclP->aclVersion);
   ???*/
  for (i = 0; i < count; i++)
  {
    if (tmpaclP->aclVersion == ACL_SECOND_VERSION)
    {
      switch(aceP->aceTagType)
      {
        case ACL_USER:
          aclP->a_entries[i].e_tag = LINUX_ACL_USER;
          aclP->a_entries[i].e_perm = PERM_MASK(aceP->acePermissions);
          aclP->a_entries[i].e_id = aceP->aceQualifier;
        case ACL_GROUP:
          aclP->a_entries[i].e_tag = LINUX_ACL_GROUP;
          aclP->a_entries[i].e_perm = PERM_MASK(aceP->acePermissions);
          aclP->a_entries[i].e_id = aceP->aceQualifier;
        case ACL_USER_OBJ:
          aclP->a_entries[i].e_tag = LINUX_ACL_USER_OBJ;
          aclP->a_entries[i].e_perm = PERM_MASK(aceP->acePermissions);
          aclP->a_entries[i].e_id = LINUX_ACL_UNDEFINED_ID;
        case ACL_GROUP_OBJ:
          aclP->a_entries[i].e_tag = LINUX_ACL_GROUP_OBJ;
          aclP->a_entries[i].e_perm = PERM_MASK(aceP->acePermissions);
          aclP->a_entries[i].e_id = LINUX_ACL_UNDEFINED_ID;
        case ACL_OTHER:
          aclP->a_entries[i].e_tag = LINUX_ACL_OTHER;
          aclP->a_entries[i].e_perm = PERM_MASK(aceP->acePermissions);
          aclP->a_entries[i].e_id = LINUX_ACL_UNDEFINED_ID;
        case ACL_MASK:
          aclP->a_entries[i].e_tag = LINUX_ACL_MASK;
          aclP->a_entries[i].e_perm = PERM_MASK(aceP->acePermissions);
          aclP->a_entries[i].e_id = LINUX_ACL_UNDEFINED_ID;
      }
    }
    else
    {
    /*???*/
    }
    /* move to the next acl entry */
    aceP = (acl_entry_t *)((caddr_t)aceP + aceP->aceLength);
  }
}
#endif /* NFS4_CONFIG_FS_POSIX_ACL */
#endif /* NFS4_CONFIG_FS_EXT_ATTR */
